import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { AdCreatedDomainEvent } from '../../domain/events/ad-created.domain-events';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { AD_MESSAGE_PUBLISHER } from '@modules/ad/ad.di-tokens';
import { AD_CREATED_ROUTING_KEY } from '@src/app.constants';

@Injectable()
export class PublishMessageWhenAdIsCreatedDomainEventHandler {
  constructor(
    @Inject(AD_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(AdCreatedDomainEvent.name, { async: true, promisify: true })
  async handle(event: AdCreatedDomainEvent): Promise<any> {
    this.messagePublisher.publish(
      AD_CREATED_ROUTING_KEY,
      JSON.stringify(event),
    );
  }
}
