import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { v4 } from 'uuid';
import { AdCreatedDomainEvent } from './events/ad-created.domain-events';
import { AdProps, CreateAdProps } from './ad.types';
import { ScheduleItemProps } from './value-objects/schedule-item.value-object';
import { WaypointProps } from './value-objects/waypoint.value-object';

export class AdEntity extends AggregateRoot<AdProps> {
  protected readonly _id: AggregateID;

  static create = (create: CreateAdProps): AdEntity => {
    const id = v4();
    const props: AdProps = { ...create };
    const ad = new AdEntity({ id, props });
    ad.addEvent(
      new AdCreatedDomainEvent({
        aggregateId: id,
        userId: props.userId,
        driver: props.driver,
        passenger: props.passenger,
        frequency: props.frequency,
        fromDate: props.fromDate,
        toDate: props.toDate,
        schedule: props.schedule.map((day: ScheduleItemProps) => ({
          day: day.day as number,
          time: day.time,
          margin: day.margin as number,
        })),
        seatsProposed: props.seatsProposed,
        seatsRequested: props.seatsRequested,
        strict: props.strict,
        waypoints: props.waypoints.map((waypoint: WaypointProps) => ({
          position: waypoint.position,
          name: waypoint.address.name,
          houseNumber: waypoint.address.houseNumber,
          street: waypoint.address.street,
          postalCode: waypoint.address.postalCode,
          locality: waypoint.address.locality,
          country: waypoint.address.country,
          lon: waypoint.address.coordinates.lon,
          lat: waypoint.address.coordinates.lat,
        })),
      }),
    );
    return ad;
  };

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
